/* Display.h
*/

#ifndef DISPLAY_H
#define DISPLAY_H

#ifdef __cplusplus  /* Allow header file to be included in a C++ file */
extern "C" {
#endif

#include "service.h"
#include "sme_rc.h"
#include "UICtrl.h"

/*----------------------------------------------------------------------------*/

#define DISPLAY_SCREEN_SIZE_WIDTH		96
#define DISPLAY_SCREEN_SIZE_HEIGHT		64
#define DISPLAY_MENU_SIZE_WIDTH			88	
#define	DISPLAY_MENU_SIZE_HEIGHT		48
#define DISPLAY_IMAGE_MENU_SIZE_WIDTH	88	
#define	DISPLAY_IMAGE_MENU_SIZE_HEIGHT	16
#define	DISPLAY_SOFTKEY_SIZE_TOP		48
#define	DISPLAY_SOFTKEY_SIZE_WIDTH		40
#define DISPLAY_SOFTKEY_SIZE_HEIGHT		16
#define DISPLAY_SCROLL_SIZE_WIDTH		8
#define DISPLAY_TEXTAREA_SIZE_WIDTH		(DISPLAY_SCREEN_SIZE_WIDTH-DISPLAY_SCROLL_SIZE_WIDTH)
#define DISPLAY_TEXTAREA_SIZE_HEIGHT	(DISPLAY_SCREEN_SIZE_HEIGHT-DISPLAY_SOFTKEY_SIZE_HEIGHT)

/*If you want to support a higher level,modify here*/
#define DISPLAY_SUPPORT_COLOR_NUM		256
#define DISPLAY_BIT_PER_PIXEL			8
#define DISPLAY_ADJUST_NUMBER			(DISPLAY_SUPPORT_COLOR_NUM-1)
#define DISPLAY_BYTE_PER_LINE			(12*DISPLAY_BIT_PER_PIXEL)
#define DISPLAY_BIT_PER_BYTE			8

#define DISPLAY_DEFAULT_PEN_COLOR		0x00
#define DISPLAY_DEFAULT_BG_COLOR		0xff
#define DISPLAY_CTRL_BG_COLOR_BL_OFF	0xf1ddb4
#define DISPLAY_CTRL_BG_COLOR_BL_ON		0xf1eedd

#define POINTER_OF_MENU_DATA			SME_MENU_DATA_T*
#define DEFAULT_DIGIT_FONT				0

typedef int DISPLAY_COLOR_T;
typedef int DISPLAY_COORDINATE_T;
typedef int DISPLAY_LENGTH_T;

typedef struct  {
    DISPLAY_COORDINATE_T x;
    DISPLAY_COORDINATE_T y;
} DISPLAY_POINT_T;

typedef struct  {
    DISPLAY_LENGTH_T     w;
    DISPLAY_LENGTH_T     h;
} DISPLAY_AREA_SIZE_T;

typedef struct  {
    DISPLAY_POINT_T      from;
    DISPLAY_AREA_SIZE_T  size;
} DISPLAY_AREA_T;

typedef unsigned char DISPLAY_BUFFER[64*DISPLAY_BYTE_PER_LINE];

enum{
	MODE_CENTER,
	MODE_LEFT,
	MODE_RIGHT,
};

DISPLAY_BUFFER Buf;

/*----------------------------------------------------------------------------*/

BOOL SetBit(unsigned char* pU8,int bit_pos);

BOOL SetByte(unsigned char* pU8,const unsigned char* pData);

BOOL GetData(unsigned char* pU8,int data_pos,
			 DISPLAY_COLOR_T* pColor);

BOOL SetData(unsigned char* pU8,int data_pos,
			 DISPLAY_COLOR_T color);

BOOL SnapShot(DISPLAY_BUFFER Data);

BOOL ClearScreen();

BOOL ClearRectangle(DISPLAY_COORDINATE_T x,
				    DISPLAY_COORDINATE_T y, 
				    DISPLAY_COORDINATE_T width,
					DISPLAY_COORDINATE_T height);

BOOL SetRectangle(DISPLAY_COORDINATE_T x,
				     DISPLAY_COORDINATE_T y, 
				     DISPLAY_COORDINATE_T width,
					 DISPLAY_COORDINATE_T height,
					 DISPLAY_BUFFER Data);

unsigned char* GetPointHandler(DISPLAY_COORDINATE_T x,
							   DISPLAY_COORDINATE_T y,
							   int* pDataPos);

BOOL DrawPoint(DISPLAY_COORDINATE_T x,
			   DISPLAY_COORDINATE_T y,
			   DISPLAY_COLOR_T color);


BOOL DrawVertical(DISPLAY_COORDINATE_T x,
				  DISPLAY_COORDINATE_T y0,
				  DISPLAY_COORDINATE_T y1,
				  DISPLAY_COLOR_T color);

BOOL DrawHorizontal(DISPLAY_COORDINATE_T x0,
					DISPLAY_COORDINATE_T x1,
					DISPLAY_COORDINATE_T y,
					DISPLAY_COLOR_T color);

BOOL DrawLine(DISPLAY_COORDINATE_T x0,
			  DISPLAY_COORDINATE_T x1,
			  DISPLAY_COORDINATE_T y0,
			  DISPLAY_COORDINATE_T y1,
			  DISPLAY_COLOR_T color);

BOOL DrawRectangle(DISPLAY_COORDINATE_T x0,
				   DISPLAY_COORDINATE_T y0,
				   DISPLAY_COORDINATE_T x1,
				   DISPLAY_COORDINATE_T y1,
				   DISPLAY_COLOR_T color);

BOOL DrawFontBitmap(DISPLAY_COORDINATE_T x,
					DISPLAY_COORDINATE_T y,
					unsigned long nFontID,
					unsigned short nChar,
					DISPLAY_COLOR_T color);

BOOL DrawCharacterAt(int line,
					 int num,
					 unsigned long nFontID,
					 unsigned short nChar,
					 DISPLAY_COLOR_T color);

BOOL DrawDigit(DISPLAY_COORDINATE_T x,
			   DISPLAY_COORDINATE_T y,
			   int i,
			   DISPLAY_COLOR_T color);

BOOL DrawNumber(DISPLAY_COORDINATE_T x,
			    DISPLAY_COORDINATE_T y,
			    int num,
			    DISPLAY_COLOR_T color);

int GetFontHeight(unsigned long nFontID);

int GetFontWidth(unsigned long nFontID);

const SME_CHAR* GetTextStr(unsigned long nRcId, unsigned char nLang);

int GetStrLen(const SME_CHAR* pText);

unsigned short GetStrCh(const SME_CHAR* pText,int ch_pos);

BOOL DrawTextStr(DISPLAY_COORDINATE_T x,
				 DISPLAY_COORDINATE_T y,
				 const SME_CHAR* pStr,
				 unsigned long nFontID,
				 DISPLAY_COLOR_T color);

BOOL DrawTextStrAt(int line,
				   int mode,
				   const SME_CHAR* pStr,
				   unsigned long nFontID,
				   DISPLAY_COLOR_T color);

BOOL DrawMenu(unsigned long nRcId,
			  unsigned char nLang,unsigned long nFontID,int* pPos,
			  DISPLAY_COLOR_T color);

BOOL DrawImageMenu(unsigned long nMenuId,unsigned long nBitmapId,
				   unsigned char nLang,unsigned long nFontID,int pos,DISPLAY_COLOR_T color);

int GetMenuData(unsigned long nRcId,int menu_pos,
						   POINTER_OF_MENU_DATA* pMenuData);

BOOL DrawReverseArea(DISPLAY_COORDINATE_T byte_x,
					 DISPLAY_COORDINATE_T pixel_y,
					 DISPLAY_COORDINATE_T byte_width,
					 DISPLAY_COORDINATE_T pixel_height,
					 BOOL mode);

BOOL SetMenuFocus(unsigned long nRcId,
				  unsigned long nFontID,int old_pos,int new_pos);

int GetBitmapHeight(unsigned long nRcId);

int GetBitmapWidth(unsigned long nRcId);

BOOL DrawBitmap(DISPLAY_COORDINATE_T x,
				DISPLAY_COORDINATE_T y,
				unsigned long nRcId);

BOOL DrawBitmapAt(int line,
				  int mode,
				  unsigned long nFontID,
				  unsigned long nRcId);

BOOL DrawSoftkey(const SOFTKEY_INFO_T* pSoftkey,
				 unsigned char nLang,
				 unsigned long nFontId,
				 DISPLAY_COLOR_T color);

/*----------------------------------------------------------------------------*/

#ifdef __cplusplus  /* Allow header file to be included in a C++ file */
}
#endif /* __cplusplus */

#endif  /* DISPLAY_H  */