/*
FILE: MenuCtrl.c
NOTE: The State Machine Wizard will add mapping macros between /*{{ and /*}}. Do NOT modify manually.
*/

#include "sme_debug.h"
#include "Service.h"
#include "display.h"
#include "MenuCtrl.h"
#include "global.h"

SME_BEGIN_STATE_DEF(MenuCtrl,MenuCtrl)
	/*{{SME_STATE_DEF(MenuCtrl,MenuCtrl)*/
	SME_STATE_ENTRY_FUNC(MenuCtrlEntry)
	SME_STATE_EXIT_FUNC(MenuCtrlExit)
	SME_ON_EVENT(EVENT_KEY_LEFTSOFT,OnMenuCtrlEVENT_KEY_LEFTSOFT,SME_INTERNAL_TRAN)
	SME_ON_EVENT(EVENT_KEY_RIGHTSOFT,OnMenuCtrlEVENT_KEY_RIGHTSOFT,SME_INTERNAL_TRAN)
	SME_ON_EVENT(EVENT_KEY_UP,OnMenuCtrlEVENT_KEY_UP,SME_INTERNAL_TRAN)
	SME_ON_EVENT(EVENT_KEY_DOWN,OnMenuCtrlEVENT_KEY_DOWN,SME_INTERNAL_TRAN)
	SME_ON_EVENT(EVENT_MENU_OK,OnMenuCtrlEVENT_MENU_OK,SME_INTERNAL_TRAN)
	SME_ON_EVENT(EVENT_MENU_CANCEL,OnMenuCtrlEVENT_MENU_CANCEL,SME_INTERNAL_TRAN)
	/*}}SME_STATE_DEF*/
SME_END_STATE_DEF

/*{{SME_STATE_STATETREE_SEPARATOR}}*/

SME_BEGIN_STATE_TREE_DEF(MenuCtrl)
	/*{{SME_STATE_TREE_DEF(MenuCtrl)*/
	SME_STATE(MenuCtrl,MenuCtrl,SME_INVALID_STATE,-1)
	/*}}SME_STATE_TREE_DEF*/
SME_END_STATE_TREE_DEF

/*{{SME_DEC_IMP_SEPARATOR}}*/

SME_APPLICATION_DEF(MenuCtrl, "MenuCtrl")

#define	DEFAULT_MAX_MENU_LAYER		16

int layer;/*outer menu == 0*/
long root_menu_id[DEFAULT_MAX_MENU_LAYER];
int current_page[DEFAULT_MAX_MENU_LAYER];
int surf_pos[DEFAULT_MAX_MENU_LAYER];
int item_per_page,item_in_root_menu,item_in_current_page,page_num;

void* pBlockForMenu=NULL;

int MenuCtrlEntry(struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	POINTER_OF_MENU_DATA pMenuData;
	int item_count=0;
	int i=0;

	pBlockForMenu=SmeMAlloc(32);

	for(i=0;i<DEFAULT_MAX_MENU_LAYER;i++){
		root_menu_id[i]=-1;
		current_page[i]=0;
		surf_pos[i]=0;
	}
	layer=0;
	root_menu_id[layer]=default_topmenu;
	item_per_page=DISPLAY_MENU_SIZE_HEIGHT/GetFontHeight(default_font);
	item_in_root_menu=GetMenuData(root_menu_id[layer],0,&pMenuData);
	page_num=0;
	while(item_count<item_in_root_menu){
		item_count+=item_per_page;
		page_num++;
	}
	item_in_current_page=0;

	MenuCtrlUpdateDisplay();
	UICtrlSetSoftkey(&SoftKey,SID_MENU_SKL,EVENT_MENU_CANCEL,SID_MENU_SKR,EVENT_MENU_OK);
	DrawSoftkey(&SoftKey,default_lang,default_font,DISPLAY_DEFAULT_PEN_COLOR);
	SrvUpdateDisplay(0,DISPLAY_SOFTKEY_SIZE_TOP,96,DISPLAY_SOFTKEY_SIZE_HEIGHT,Buf);

	return 0;
}

int MenuCtrlExit(struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	SmeMFree(pBlockForMenu);
	return 0;
}

int MenuCtrlUpdateDisplay()
{
	int pivot_pos=current_page[layer]*item_per_page;
	int up,down;
	ClearRectangle(0,0,DISPLAY_SCREEN_SIZE_WIDTH,DISPLAY_TEXTAREA_SIZE_HEIGHT);
	DrawBitmap(88,0,IDB_ROLL);
	DrawMenu(root_menu_id[layer],default_lang,default_font,&pivot_pos,DISPLAY_DEFAULT_PEN_COLOR);
	SetMenuFocus(root_menu_id[layer],default_font,-1,surf_pos[layer]);
	item_in_current_page=pivot_pos;
	up=9+(int)((float)current_page[layer]/page_num*29);
	down=38-(int)((float)(page_num-current_page[layer]-1)/page_num*29);
	DrawRectangle(88,up,95,down,0xfc);
	SrvUpdateDisplay(0,0,96,DISPLAY_TEXTAREA_SIZE_HEIGHT,Buf);
	return 0;
}

int OnMenuCtrlEVENT_KEY_LEFTSOFT(struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	UICtrlLeftSoftKeyPress(&SoftKey);
	return 0;
}

int OnMenuCtrlEVENT_KEY_RIGHTSOFT(struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	UICtrlRightSoftKeyPress(&SoftKey);
	return 0;
}

int OnMenuCtrlEVENT_KEY_UP (struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	if(surf_pos[layer]>0){
		surf_pos[layer]--;
		SetMenuFocus(root_menu_id[layer],default_font,surf_pos[layer]+1,surf_pos[layer]);
		SrvUpdateDisplay(0,0,96,64,Buf);
	}
	else if(current_page[layer]>0)
		MenuPrePage();
	return 0;
}

int OnMenuCtrlEVENT_KEY_DOWN (struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	if(surf_pos[layer]<item_in_current_page-1){
		surf_pos[layer]++;
		SetMenuFocus(root_menu_id[layer],default_font,surf_pos[layer]-1,surf_pos[layer]);
		SrvUpdateDisplay(0,0,96,64,Buf);
	}
	else if(current_page[layer]<page_num-1)
		MenuNextPage();
	return 0;
}

int MenuPrePage()
{
	int pivot_pos=0;
	if(current_page[layer]>0){
		pivot_pos=(current_page[layer]-1)*item_per_page;
		surf_pos[layer]=item_per_page-1;
		current_page[layer]--;
		MenuCtrlUpdateDisplay();
	}
	return 0;
}

int MenuNextPage()
{
	int pivot_pos=0;
	if(current_page[layer]<page_num-1){
		pivot_pos=(current_page[layer]+1)*item_per_page;
		surf_pos[layer]=0;
		current_page[layer]++;
		MenuCtrlUpdateDisplay();
	}
	return 0;
}

int OnMenuCtrlEVENT_MENU_OK (struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	POINTER_OF_MENU_DATA pMenuData;
	int item_count=0;
	SME_EVENT_T *pInteralEvent;
	GetMenuData(root_menu_id[layer],surf_pos[layer]+current_page[layer]*item_per_page,&pMenuData);
	if(pMenuData->nSubMenuID>=0){
		layer++;
		root_menu_id[layer]=pMenuData->nSubMenuID;
		current_page[layer]=0;
		surf_pos[layer]=0;
		item_in_root_menu=GetMenuData(root_menu_id[layer],0,&pMenuData);
		page_num=0;
		while(item_count<item_in_root_menu){
			item_count+=item_per_page;
			page_num++;
		}
		MenuCtrlUpdateDisplay();
	}
	else{
		pInteralEvent=SmeCreateIntEvent(EVENT_MENU_SEL,pMenuData->nMenuTitleID,0,SME_EVENT_CAT_UI,pApp->pParent);
		SmePostEvent(pInteralEvent);
	}
	return 0;
}

int OnMenuCtrlEVENT_MENU_CANCEL (struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	POINTER_OF_MENU_DATA pMenuData;
	int item_count=0;
	root_menu_id[layer]=-1;
	current_page[layer]=0;
	surf_pos[layer]=0;
	if(layer>0){
		layer--;
		item_in_root_menu=GetMenuData(root_menu_id[layer],0,&pMenuData);
		page_num=0;
		while(item_count<item_in_root_menu){
			item_count+=item_per_page;
			page_num++;
		}
		MenuCtrlUpdateDisplay();
	}
	else
		UICtrlPostInternalEvent(EVENT_MENU_EXIT,pApp->pParent);
	return 0;
}

int OnMenuCtrlEVENT_ON_SELECT (struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	return 0;
}


int OnMenuCtrlEVENT_ON_EXIT (struct SME_APP_T *pApp, struct SME_EVENT_T *pEvent)
{
	return 0;
}

